@@ -10,6 +10,7 @@ class SqtVersion(Enum):
     V1_4_0 = auto()
     V2_1_0 = auto()
     V2_1_0_ext = auto()
+    V2_1_0_robin_random = auto()
 
 
 @dataclass
@@ -22,6 +23,7 @@ class LLine:
 l_line_V1_4_0_template = 'L\t{locus_name}\t{peptide_index_in_protein_sequence}\t{peptide_sequence}\n'
 l_line_V2_1_0_template = 'L\t{locus_name}\t{peptide_index_in_protein_sequence}\t{peptide_sequence}\n'
 l_line_V2_1_0_ext_template = 'L\t{locus_name}\t{peptide_index_in_protein_sequence}\t{peptide_sequence}\n'
+l_line_V2_1_0_robin_random_template = 'L\t{locus_name}\t{peptide_index_in_protein_sequence}\t{peptide_sequence}\n'
 
 
 def _deserialize_l_line(line: str, sqt_version: SqtVersion) -> LLine:
@@ -41,6 +43,11 @@ def _deserialize_l_line(line: str, sqt_version: SqtVersion) -> LLine:
                      peptide_index_in_protein_sequence=deserialize_val(line_elems[2], int),
                      peptide_sequence=deserialize_val(line_elems[3], str),
                      )
+    elif sqt_version == SqtVersion.V2_1_0_robin_random:
+        return LLine(locus_name=deserialize_val(line_elems[1], str),
+                     peptide_index_in_protein_sequence=deserialize_val(line_elems[2], int),
+                     peptide_sequence=deserialize_val(line_elems[3], str),
+                     )
     else:
         raise ValueError(f'Unsupported Sqt Version: {sqt_version}!')
 
@@ -64,6 +71,12 @@ def _serialize_l_line(line: LLine, sqt_version: SqtVersion) -> str:
             peptide_index_in_protein_sequence=serialize_val(line.peptide_index_in_protein_sequence),
             peptide_sequence=serialize_val(line.peptide_sequence),
         )
+    elif sqt_version == SqtVersion.V2_1_0_robin_random:
+        return l_line_V2_1_0_robin_random_template.format(
+            locus_name=serialize_val(line.locus_name),
+            peptide_index_in_protein_sequence=serialize_val(line.peptide_index_in_protein_sequence),
+            peptide_sequence=serialize_val(line.peptide_sequence),
+        )
     else:
         raise ValueError(f'Unsupported Sqt Version: {sqt_version}!')
 
@@ -95,7 +108,9 @@ m_line_V2_1_0_template = 'M\t{xcorr_rank}\t{sp_rank}\t{calculated_mass}\t{delta_
 m_line_V2_1_0_ext_template = 'M\t{xcorr_rank}\t{sp_rank}\t{calculated_mass}\t{delta_cn}\t{xcorr}\t{sp}' \
                              '\t{matched_ions}\t{expected_ions}\t{sequence}\t{validation_status}\t{predicted_ook0}' \
                              '\t{tims_score}\t{tims_b_score_m2}\t{tims_b_score_best_m}\n'
-
+m_line_V2_1_0_robin_random_template = 'M\t{xcorr_rank}\t{sp_rank}\t{calculated_mass}\t{delta_cn}\t{xcorr}\t{sp}' \
+                             '\t{matched_ions}\t{expected_ions}\t{sequence}\t{validation_status}\t{predicted_ook0}' \
+                             '\t{tims_score}\n'
 
 def _deserialize_m_line(line: str, sqt_version: SqtVersion) -> MLine:
     line_elems = line.rstrip().split('\t')
@@ -150,6 +165,23 @@ def _deserialize_m_line(line: str, sqt_version: SqtVersion) -> MLine:
             tims_b_score_m2=deserialize_val(line_elems[13], float),
             tims_b_score_best_m=deserialize_val(line_elems[14], float),
         )
+    elif sqt_version == SqtVersion.V2_1_0_robin_random:
+        return MLine(
+            xcorr_rank=deserialize_val(line_elems[1], int),
+            sp_rank=deserialize_val(line_elems[2], int),
+            calculated_mass=deserialize_val(line_elems[3], float),
+            delta_cn=deserialize_val(line_elems[4], float),
+            xcorr=deserialize_val(line_elems[5], float),
+            sp=deserialize_val(line_elems[6], float),
+            matched_ions=deserialize_val(line_elems[7], int),
+            expected_ions=deserialize_val(line_elems[8], int),
+            sequence=deserialize_val(line_elems[9], str),
+            validation_status=deserialize_val(line_elems[10], str),
+            predicted_ook0=deserialize_val(line_elems[11], float),
+            tims_score=deserialize_val(line_elems[12], float),
+            tims_b_score_m2=None,
+            tims_b_score_best_m=None
+        )
     else:
         raise ValueError(f'Unsupported Sqt Version: {sqt_version}!')
 
@@ -200,6 +232,21 @@ def _serialize_m_line(line: MLine, sqt_version: SqtVersion) -> str:
             tims_b_score_m2=serialize_val(line.tims_b_score_m2, 4),
             tims_b_score_best_m=serialize_val(line.tims_b_score_best_m, 4),
         )
+    elif sqt_version == SqtVersion.V2_1_0_robin_random:
+        return m_line_V2_1_0_robin_random_template.format(
+            xcorr_rank=serialize_val(line.xcorr_rank),
+            sp_rank=serialize_val(line.sp_rank),
+            calculated_mass=serialize_val(line.calculated_mass, 5),
+            delta_cn=serialize_val(line.delta_cn, 4),
+            xcorr=serialize_val(line.xcorr, 4),
+            sp=serialize_val(line.sp, 3),
+            matched_ions=serialize_val(line.matched_ions),
+            expected_ions=serialize_val(line.expected_ions),
+            sequence=serialize_val(line.sequence),
+            validation_status=serialize_val(line.validation_status),
+            predicted_ook0=serialize_val(line.predicted_ook0, 4),
+            tims_score=serialize_val(line.tims_score, 4),
+        )
     else:
         raise ValueError(f'Unsupported Sqt Version: {sqt_version}!')
 
@@ -229,7 +276,9 @@ s_line_V2_1_0_template = 'S\t{low_scan}\t{high_scan}\t{charge}\t{process_time}\t
 s_line_V2_1_0_ext_template = 'S\t{low_scan}\t{high_scan}\t{charge}\t{process_time}\t{server}\t{experimental_mass}' \
                              '\t{total_ion_intensity}\t{lowest_sp}\t{number_matches}\t{experimental_ook0}' \
                              '\t{experimental_mz}\t{corrected_ook0}\n'
-
+s_line_V2_1_0_robin_random_template = 'S\t{low_scan}\t{high_scan}\t{charge}\t{process_time}\t{server}\t{experimental_mass}' \
+                             '\t{total_ion_intensity}\t{lowest_sp}\t{number_matches}\t{experimental_ook0}' \
+                             '\t{experimental_mz}\n'
 
 def _deserialize_s_line(line: str, sqt_version: SqtVersion) -> SLine:
     line_elems = line.rstrip().split('\t')
@@ -278,10 +327,24 @@ def _deserialize_s_line(line: str, sqt_version: SqtVersion) -> SLine:
             experimental_mz=deserialize_val(line_elems[11], float),
             corrected_ook0=deserialize_val(line_elems[12], float),
         )
+    elif sqt_version == SqtVersion.V2_1_0_robin_random:
+        return SLine(
+            low_scan=deserialize_val(line_elems[1], int),
+            high_scan=deserialize_val(line_elems[2], int),
+            charge=deserialize_val(line_elems[3], int),
+            process_time=deserialize_val(line_elems[4], int),
+            server=deserialize_val(line_elems[5], str),
+            experimental_mass=deserialize_val(line_elems[6], float),
+            total_ion_intensity=deserialize_val(line_elems[7], float),
+            lowest_sp=deserialize_val(line_elems[8], float),
+            number_matches=deserialize_val(line_elems[9], int),
+            experimental_ook0=deserialize_val(line_elems[10], float),
+            experimental_mz=deserialize_val(line_elems[11], float),
+            corrected_ook0=None
+        )
     else:
         raise ValueError(f'Unsupported Sqt Version: {sqt_version}!')
 
-
 def _serialize_s_line(line: SLine, sqt_version: SqtVersion) -> str:
     if sqt_version == SqtVersion.V1_4_0:
         return s_line_V1_4_0_template.format(low_scan=serialize_val(line.low_scan),
@@ -320,6 +383,19 @@ def _serialize_s_line(line: SLine, sqt_version: SqtVersion) -> str:
                                                  experimental_mz=serialize_val(line.experimental_mz, 4),
                                                  corrected_ook0=serialize_val(line.corrected_ook0, 4),
                                                  )
+    elif sqt_version == SqtVersion.V2_1_0_robin_random:
+        return s_line_V2_1_0_robin_random_template.format(low_scan=serialize_val(line.low_scan),
+                                                 high_scan=serialize_val(line.high_scan),
+                                                 charge=serialize_val(line.charge),
+                                                 process_time=serialize_val(line.process_time),
+                                                 server=serialize_val(line.server),
+                                                 experimental_mass=serialize_val(line.experimental_mass, 5),
+                                                 total_ion_intensity=serialize_val(line.total_ion_intensity, 2),
+                                                 lowest_sp=serialize_val(line.lowest_sp, 4),
+                                                 number_matches=serialize_val(line.number_matches),
+                                                 experimental_ook0=serialize_val(line.experimental_ook0, 4),
+                                                 experimental_mz=serialize_val(line.experimental_mz, 4),
+                                                 )
     else:
         raise ValueError(f'Unsupported Sqt Version: {sqt_version}!')
 
@@ -333,6 +409,8 @@ def determine_sqt_version(s_line: str) -> SqtVersion:
         return SqtVersion.V2_1_0
     elif len(line_elems) == 13:
         return SqtVersion.V2_1_0_ext
+    elif len(line_elems) == 12:
+        return SqtVersion.V2_1_0_robin_random
     else:
         raise ValueError(f'Cannot parse version from s_line: {s_line}!')
 
